// Copyright Joyent, Inc. and other Node contributors.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.

var util = require('util.js');
var fs = require('fs');

var logFile = null;

exports.setLog = function(name) {
  try {
	  logFile = fs.openSync(name, 'w');
	  fs.write(logFile,"Open Log At "+(new Date())+"\n");
	  process.on('exit', function() {
		  try{fs.close(logFile)}catch(e){};
		});
	}catch (e) {}
};
exports.log = function() {
	var a = util.format.apply(this, arguments) + '\n';
  process.stdout.write(a);
  if(logFile)
  	fs.write(logFile,(new Date()).format("d-m-Y H:i:s - ")+"LOG : "+a);
};


exports.info = function() {
  var a = util.format.apply(this, arguments) + '\n';
  process.stdout.write(util.colorStr(a,'blue'));
  if(logFile)
  	fs.write(logFile,(new Date()).format("d-m-Y H:i:s - ")+"INFO : "+a);
};;


exports.warn = function() {
	var a  = util.format.apply(this, arguments) + '\n';
  process.stderr.write(util.colorStr(a,'yellow'));
  if(logFile)
  	fs.write(logFile,(new Date()).format("d-m-Y H:i:s - ")+"WARN : "+a);
};


exports.error = function() {
	var a = util.format.apply(this, arguments) + '\n';
  process.stderr.write(util.colorStr(a,'red'));
  if(logFile)
  	fs.write(logFile,(new Date()).format("d-m-Y H:i:s - ")+"ERR : "+a);
};


exports.dir = function(object) {
  process.stdout.write(util.inspect(object) + '\n');
  if(logFile)
  	fs.write(logFile,(new Date()).format("d-m-Y H:i:s - ")+"DIR : "+util.format.apply(this, arguments) + '\n');
};


var times = {};
exports.time = function(label) {
  times[label] = Date.now();
};


exports.timeEnd = function(label) {
  var duration = Date.now() - times[label];
  exports.log('%s: %dms', label, duration);
};


exports.trace = function(label) {
  // TODO probably can to do this better with V8's debug object once that is
  // exposed.
  var err = new Error;
  err.name = 'Trace';
  err.message = label || '';
  Error.captureStackTrace(err, arguments.callee);
  console.error(err.stack);
};


exports.assert = function(expression) {
  if (!expression) {
    var arr = Array.prototype.slice.call(arguments, 1);
    require('assert').ok(false, util.format.apply(this, arr));
  }
};
